@@ -5,7 +5,7 @@ use warnings;
 package Sub::MultiMethod;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.907';
+our $VERSION   = '0.908';
 
 use B ();
 use Exporter::Shiny qw(
@@ -41,17 +41,19 @@ sub get_multimethods {
 
 sub _get_multimethod_candidates_ref {
 	my ($me, $target, $method_name) = @_;
-	my $method_key  = ref($method_name) ? refaddr($method_name) : $method_name;
-	my $package_key = is_Int($method_key) ? '__CODE__' : $target;
-	my $mm = $me->_get_multimethods_ref($package_key);
+	my ( $package_key, $method_key ) = ref( $method_name )
+		? ( '__CODE__', refaddr( $method_name ) )
+		: ( $target, $method_name );
+	my $mm = $me->_get_multimethods_ref( $package_key );
 	$mm->{$method_key} ||= [];
 }
 
 sub _clear_multimethod_candidates_ref {
-	my ($me, $target, $method_name) = @_;
-	my $method_key  = ref($method_name) ? refaddr($method_name) : $method_name;
-	my $package_key = is_Int($method_key) ? '__CODE__' : $target;
-	my $mm = $me->_get_multimethods_ref($package_key);
+	my ( $me, $target, $method_name ) = ( shift, @_ );
+	my ( $package_key, $method_key ) = ref( $method_name )
+		? ( '__CODE__', refaddr( $method_name ) )
+		: ( $target, $method_name );
+	my $mm = $me->_get_multimethods_ref( $package_key );
 	delete $mm->{$method_key};
 	return $me;
 }
@@ -69,8 +71,7 @@ sub has_multimethod_candidates {
 sub _add_multimethod_candidate {
 	my ($me, $target, $method_name, $spec) = @_;
 	my $mmc = $me->_get_multimethod_candidates_ref($target, $method_name);
-	push @$mmc, $spec;
-	if ($spec->{method} != $mmc->[0]{method}) {
+	if ( @$mmc and $spec->{method} != $mmc->[0]{method} ) {
 		require Carp;
 		Carp::carp(sprintf(
 			"Added multimethod candidate for %s with method=>%d but expected method=>%d",
@@ -79,6 +80,7 @@ sub _add_multimethod_candidate {
 			$mmc->[0]{method},
 		));
 	}
+	push @$mmc, $spec;
 	$me;
 }
 
